@@ -1,7 +1,7 @@
 /*
  * image viewer, for framebuffer devices
  *
- *   (c) 1998-2004 Gerd Hoffmann <kraxel@bytesex.org>
+ *   (c) 1998-2012 Gerd Hoffmann <gerd@kraxel.org>
  *
  */
 
@@ -43,7 +43,7 @@
 #include "list.h"
 #include "fbiconfig.h"
 
-#include "jpeg/transupp.h"		/* Support routines for jpegtran */
+#include "transupp.h"		/* Support routines for jpegtran */
 #include "jpegtools.h"
 
 #define TRUE            1
@@ -77,6 +77,8 @@
 #define KEY_DELETE   -200
 #define KEY_ROT_CW   -201
 #define KEY_ROT_CCW  -202
+#define KEY_FLIP_V   -203
+#define KEY_FLIP_H   -204
 
 #define DEFAULT_DEVICE  "/dev/fb0"
 
@@ -162,7 +164,7 @@ version(void)
 {
     fprintf(stderr,
 	    "fbi version " VERSION ", compiled on %s\n"
-	    "(c) 1999-2006 Gerd Hoffmann <kraxel@bytesex.org> [SUSE Labs]\n",
+	    "(c) 1998-2012 Gerd Hoffmann <gerd@kraxel.org> [SUSE Labs]\n",
 	    __DATE__ );
 }
 
@@ -176,8 +178,8 @@ usage(char *name)
     fprintf(stderr,
 	    "\n"
 	    "This program displays images using the Linux framebuffer device.\n"
-	    "Supported formats: PhotoCD, jpeg, ppm, gif, tiff, xwd, bmp, png.\n"
-	    "It tries to use ImageMagick's convert for unknown file formats.\n"
+	    "Supported formats: PhotoCD, jpeg, ppm, gif, tiff, xwd, bmp, png,\n"
+	    "webp. It tries to use ImageMagick's convert for unknown file formats.\n"
 	    "\n"
 	    "usage: %s [ options ] file1 file2 ... fileN\n"
 	    "\n",
@@ -191,7 +193,7 @@ usage(char *name)
 	    "Large images can be scrolled using the cursor keys.  Zoom in/out\n"
 	    "works with '+' and '-'.  Use ESC or 'q' to quit.  Space and PgDn\n"
 	    "show the next, PgUp shows the previous image. Jumping to a image\n"
-	    "works with <number>g.  Return acts like Space but additionally\n"
+	    "works with <i>g.  Return acts like Space but additionally prints\n"
 	    "prints the filename of the currently displayed image to stdout.\n"
 	    "\n");
 }
@@ -455,6 +457,9 @@ static void show_exif(struct flist *f)
 	0xa40c, // Subject Distance Range
 	0xa405, // Focal Length In 35mm Film
 	0x9209, // Flash
+
+	0xa002, // Pixel X Dimension
+	0xa003, // Pixel Y Dimension
     };
     ExifData   *ed;
     ExifEntry  *ee;
@@ -529,23 +534,28 @@ static void show_help(void)
     static wchar_t *help[] = {
 	L"keyboard commands",
 	L"~~~~~~~~~~~~~~~~~",
-	L"  ESC, Q      - quit",
-	L"  pgdn, space - next image",
-	L"  pgup        - previous image",
-	L"  +/-         - zoom in/out",
-	L"  A           - autozoom image",
-	L"  cursor keys - scroll image",
+	L"  cursor keys    - scroll image",
+	L"  PgUp, k        - previous image",
+	L"  PgDn, SPACE, j - next image",
+	L"  <i>g           - jump to image #i",
 	L"",
-	L"  H           - show this help text",
-	L"  I           - show EXIF info",
-	L"  P           - pause slideshow",
-	L"  V           - toggle statusline",
+	L"  a              - autozoom image",
+	L"  +/-            - zoom in/out",
+	L"  <i>s           - set zoom to <i>%",
+	L"",
+	L"  ESC, q         - quit",
+	L"  v              - toggle statusline",
+	L"  h              - show this help text",
+	L"  i              - show EXIF info",
+	L"  p              - pause slideshow",
 	L"",
 	L"available if started with --edit switch,",
 	L"rotation works for jpeg images only:",
-	L"  shift+D     - delete image",
-	L"  R           - rotate clockwise",
-	L"  L           - rotate counter-clockwise",
+	L"  D, Shift+d     - delete image",
+	L"  r              - rotate 90 degrees clockwise",
+	L"  l              - rotate 90 degrees counter-clockwise",
+	L"  x              - mirror image vertically (top / bottom)",
+	L"  y              - mirror image horizontally (left to right)",
     };
 
     shadow_draw_text_box(face, 24, 16, transparency,
@@ -944,8 +954,8 @@ svga_show(struct flist *f, struct flist *prev,
 	    f->left += h_steps;
 
 	} else if (0 == strcmp(key, "\x1b[5~") ||
-		   0 == strcmp(key, "j")       ||
-		   0 == strcmp(key, "J")) {
+		   0 == strcmp(key, "k")       ||
+		   0 == strcmp(key, "K")) {
 	    if (textreading && f->top > 0) {
 		redraw = 1;
 		f->top -= f->text_steps;
@@ -955,8 +965,8 @@ svga_show(struct flist *f, struct flist *prev,
 	    }
 
 	} else if (0 == strcmp(key, "\x1b[6~") ||
-		   0 == strcmp(key, "k")       ||
-		   0 == strcmp(key, "K")       ||
+		   0 == strcmp(key, "j")       ||
+		   0 == strcmp(key, "J")       ||
 		   0 == strcmp(key, "n")       ||
 		   0 == strcmp(key, "N")) {
 	    if (textreading && f->top < (int)(img->i.height - fb_var.yres)) {
@@ -990,7 +1000,13 @@ svga_show(struct flist *f, struct flist *prev,
 	} else if (0 == strcmp(key, "l") ||
 		   0 == strcmp(key, "L")) {
 	    return KEY_ROT_CCW;
-	    
+	} else if (0 == strcmp(key, "x") ||
+		   0 == strcmp(key, "X")) {
+	    return KEY_FLIP_V;
+	} else if (0 == strcmp(key, "y") ||
+		   0 == strcmp(key, "Y")) {
+	    return KEY_FLIP_H;
+
 	} else if (0 == strcmp(key, "h") ||
 		   0 == strcmp(key, "H")) {
 	    if (!help) {
@@ -1538,6 +1554,29 @@ main(int argc, char *argv[])
 	    }
 	    break;
 	}
+	case KEY_FLIP_V:
+	case KEY_FLIP_H:
+	{
+	    if (editable) {
+		snprintf(linebuffer,sizeof(linebuffer),
+			 "mirroring %s ...",fcurrent->name);
+		status_update(linebuffer, NULL);
+		jpeg_transform_inplace
+		    (fcurrent->name,
+		     (key == KEY_FLIP_V) ? JXFORM_FLIP_V : JXFORM_FLIP_H,
+		     NULL,
+		     NULL,0,
+		     (backup   ? JFLAG_FILE_BACKUP    : 0) | 
+		     (preserve ? JFLAG_FILE_KEEP_TIME : 0) | 
+		     JFLAG_TRANSFORM_IMAGE     |
+		     JFLAG_TRANSFORM_THUMBNAIL |
+		     JFLAG_UPDATE_ORIENTATION);
+		flist_img_free(fcurrent);
+	    } else {
+		status_error("readonly mode, sorry [start with --edit?]");
+	    }
+	    break;
+	}
 	case KEY_TAGFILE:
 	    fcurrent->tag = !fcurrent->tag;
 	    /* fall throuth */
